const _ = require('lodash');
const { StashKeys } = require('../../support/constants.js');
const { parseCsvAndGroupProductsByHostname } = require('../../support/csv-support.js');
const initUserSettings = require('../../ui/user-settings.js');

const GithubSelectors = {
  Missing: '#parallax_error_text:visible,img[alt*=404]:visible',
  RenderError: '.render-notice:visible',
  SomeCsvRow: '.file table td[data-line-number]:visible',
}
const ScopedStashName = 'myStashOnlyAvailableToFetchProductList';


// ADVANCED EXAMPLE. Intentionally includes numerous concepts from the "Mastering Control" section of the Kahba tutorial.
function fetchProductList_2(runner) {
  const { kahba, prefab } = runner;
  const getScopedStash = () => kahba.task().dataStash(ScopedStashName);
  initUserSettings(runner);

  return kahba.conveyor([
    kahba.runUntilFinal(() => prefab.sendConfirmationPrompt('About to fetch CSV file containing the team shopping list.', {
      description: 'You can change its location in Settings.',
      cancelButton: false
    })),

    conveyorData => prefab.userSettings.snapshot(), // if we didn't want to merge userSettings directly into conveyorData, we could keep it on conveyorData.userSettings by instead doing: userSettings.snapshot().then(userSettings => ({ userSettings })),

    conveyorData => {
      const csvUrl = conveyorData.csvUrl;
      if (_.endsWith(csvUrl, '.csv')) {
        getScopedStash().csvUrl = csvUrl; // should you prefer using the scoped stash in the XHR alternative below
        return Promise.resolve({ csvUrl });
      } else {
        return Promise.reject({
          errorCode: 'CsvUrlRequired',
          userMessage: `Expecting URL for a CSV file`
        });
      }
    },

    kahba.if(conveyorData => Promise.resolve(_isValidGithubCsvUrl(conveyorData.csvUrl)),
      _workflowToExtractCsvUrlFromGithubPage(runner)
    ),

    kahba.waterfall([
      kahba.run(conveyorData => runner.xhr(conveyorData.csvUrl)).opts({ deepParams: 2 }), // receives conveyorData because of deepParams option
      // alternatively: () => runner.xhr(getScopedStash().csvUrl),
      // alternatively: () => runner.xhr((kahba.task().findNestedArgument('csvUrl') || {}).csvUrl')
      // alternatively: use kahba.conveyor instead of kahba.waterfall
      (xhrResponse={}, next) => next(null, { productsByHostname: parseCsvAndGroupProductsByHostname(xhrResponse.body) })
    ]),

    conveyorData => {
      // We set the result on the main kahba.stash just to keep things consistent with fetch-products.iteration-1.js.
      kahba.stash[StashKeys.AllProductsByHost] = conveyorData.productsByHostname;
      return Promise.resolve();
    },
  ], (err, conveyorData={}) => {
    // note: team-purchasing.js will catch this error and begin the workflow in manually-input-product-list.js. (Which doesn't display our userMessage because the tutorial introduces this file before it's time to cover error handling.)
    return [ err, { productsByHostname: conveyorData.productsByHostname } ];
  }).subtaskOpts({ deepParams: 1 });
}


function _workflowToExtractCsvUrlFromGithubPage(runner) {
  const { kahba } = runner;

  return kahba.waterfall([
    runner.visitThePage({
      get: 'csvUrl',
      wait: GithubSelectors.SomeCsvRow,
      abort: GithubSelectors.Missing // should we land on github's Not Found page, return an error early rather than waiting for a table to render
    }),

    kahba.if(GithubSelectors.RenderError, // if Github detects a problem in the CSV content, we won't try to parse it ourselves
      kahba.error({
        errorCode: 'CsvSyntaxError',
        userMessage: '<strong>Syntax error in your CSV!</strong> Make sure all lines in the csv file have the same number of columns/commas.'
      })),

    () => Promise.resolve({ csvUrl: $('#raw-url').prop('href') }),  // this { csvUrl } result is merged into the parent's conveyorData
  ], err => {
    if (err) {
      console.warn('Invalid Github csv page', err);
      err = err.errorCode ? err : {
        errorCode: 'InvalidProductList',
        userMessage: err.userMessage || `Unable to read Github CSV file`,
      };
      return [ err ];
    }
  });
}


function _isValidGithubCsvUrl(url) {
  return _.endsWith(url, '.csv') && /^https?:\/\/github\.com/i.test(url);
}

module.exports = fetchProductList_2;